@@ -57,7 +57,7 @@ ANDROID_SDK_ROOT = os.getenv(
 )
 
 BUILD_REGEX = re.compile(r'[1-9][0-9]{6}')
-NDK_RELEASE_REGEX = re.compile(r'r[1-9][0-9]?[a-z]?')
+NDK_RELEASE_REGEX = re.compile(r'r[1-9][0-9]?[a-z]?(?:-(?:rc|beta)[0-9]+)?')
 M2REPOSITORY_REVISION_REGEX = re.compile(r'android_m2repository_r([0-9]+)\.zip')
 
 # The sub-directory to install a given package into, assumes ANDROID_SDK_ROOT as root
@@ -652,6 +652,14 @@ def parse_platforms(url, d):
     e.g. platforms;android-29, but there are multiple releases
     available, e.g. platform-29_r05.zip, platform-29_r04.zip, etc.
 
+    The build property ro.build.version.codename with the value of REL
+    means a full release, rather than preview/beta/etc.  That value
+    was not always present, but those releases oly hve a single
+    package.
+
+    platform24_r01.zip was released twice, the first being a mistake
+    with a platform.version of 'N' in source.properties.
+
     """
     if 'source.properties' in d:
         source_properties = get_properties_dict(d['source.properties'])
@@ -659,10 +667,11 @@ def parse_platforms(url, d):
         apilevel = source_properties['androidversion.apilevel']
         # TODO this should make all versions/revisions available, not only most recent
         key = ('platforms', 'android-%s' % apilevel)
-        if key in packages:
-            packages[key] = sorted([url, packages.get(key)])[-1]
-        else:
-            packages[key] = url
+        if re.match(r'^[1-9].*', source_properties.get('platform.version', '')):
+            if key in packages:
+                packages[key] = sorted([url, packages.get(key)])[-1]
+            else:
+                packages[key] = url
 
 
 def parse_platform_tools(url, d):
@@ -773,6 +782,7 @@ def build_package_list(use_net=False):
         r.raise_for_status()
 
         if etag is None or etag != r.headers.get('etag'):
+            print('Downloading', checksums_url, 'into', str(cached_checksums))
             cached_checksums.write_bytes(r.content)
             verify(cached_checksums)
             etag_file.write_text(r.headers['etag'])
@@ -869,11 +879,8 @@ def install(to_install):
     """Install specified packages, including downloading them as needed
 
     Certain packages are installed into versioned sub-directories
-    while a couple of other are always installed into the same
-    location.  In those other cases, if that location exists, the
-    directory will be removed before installing the package.  These
-    installed packages will always contain at least
-    'source.properties'.
+    while others are always installed into the same location.  These
+    installed packages will at least always have 'source.properties'.
 
     Parameters
     ----------
@@ -889,22 +896,23 @@ def install(to_install):
     for package in to_install:
         key = tuple(package.split(';'))
         url = packages[key]
-        zipball = CACHEDIR / os.path.basename(url)
-        if not zipball.exists():
-            download_file(url, zipball)
 
         if key[0] == 'extras' and len(key) in (3, 4):
             name = ';'.join(key[:3])
         else:
             name = key[0]
 
-        package_sub_dir = INSTALL_DIRS[name]
         if len(key) > 1:
-            install_dir = ANDROID_SDK_ROOT / package_sub_dir.format(revision=key[-1])
+            install_dir = ANDROID_SDK_ROOT / INSTALL_DIRS[name].format(revision=key[-1])
         else:
-            install_dir = ANDROID_SDK_ROOT / package_sub_dir
-        if '/' not in package_sub_dir and (install_dir / 'source.properties').exists():
-            shutil.rmtree(str(install_dir))
+            install_dir = ANDROID_SDK_ROOT / INSTALL_DIRS[name]
+        if install_dir.exists():
+            continue
+
+        zipball = CACHEDIR / os.path.basename(url)
+        if not zipball.exists():
+            download_file(url, zipball)
+
         install_dir.parent.mkdir(parents=True, exist_ok=True)
         _install_zipball_from_cache(zipball, install_dir)
         _generate_package_xml(install_dir, package, url)
@@ -951,11 +959,11 @@ def _install_zipball_from_cache(zipball, install_dir):
 
     print('Installing into', install_dir)
     if len(toplevels) == 1:
-        for extracted in unzip_dir.glob('*'):
-            shutil.move(str(extracted), str(install_dir))
+        extracted = [d for d in unzip_dir.iterdir()][0]
+        shutil.move(str(extracted), str(install_dir))
     else:
         install_dir.mkdir(parents=True)
-        for extracted in unzip_dir.glob('*'):
+        for extracted in unzip_dir.iterdir():
             shutil.move(str(extracted), str(install_dir))
     if zipball.exists():
         zipball.unlink()
